@@ -138,6 +138,7 @@ class WireProtocol:
     op_rollback = 31
     op_open_blob = 35
     op_get_segment = 36
+    op_put_segment = 37
     op_close_blob = 39
     op_info_database = 40
     op_info_transaction = 42
@@ -258,7 +259,21 @@ class WireProtocol:
 
         return (db_handle, event_id, {})
 
-    def params_to_blr(self, params):
+    def _create_blob(self, trans_handle, b):
+        self._op_create_blob2(trans_handle)
+        (blob_handle, blob_id, buf) = self._op_response()
+
+        i = 0
+        while i < len(b):
+            self._op_put_segment(blob_handle, b[i:i+BLOB_SEGMENT_SIZE])
+            (h, oid, buf) = self._op_response()
+            i += BLOB_SEGMENT_SIZE
+
+        self._op_close_blob(blob_handle)
+        (h, oid, buf) = self._op_response()
+        return blob_id
+
+    def params_to_blr(self, trans_handle, params):
         "Convert parameter array to BLR and values format."
         ln = len(params) * 2
         blr = bytes([5, 2, 4, 0, ln & 255, ln >> 8])
@@ -271,11 +286,15 @@ class WireProtocol:
                 t = type(p)
             if ((PYTHON_MAJOR_VER == 2 and t == str) or
                 (PYTHON_MAJOR_VER == 3 and t == bytes)):
-                v = p
-                nbytes = len(v)
-                pad_length = ((4-nbytes) & 3)
-                v += bytes([0]) * pad_length
-                blr += bytes([14, nbytes & 255, nbytes >> 8])
+                if len(p) > MAX_CHAR_LENGTH:
+                    v = self._create_blob(trans_handle, p)
+                    blr += bytes([9, 0])
+                else:
+                    v = p
+                    nbytes = len(v)
+                    pad_length = ((4-nbytes) & 3)
+                    v += bytes([0]) * pad_length
+                    blr += bytes([14, nbytes & 255, nbytes >> 8])
             elif t == int:
                 v = bint_to_bytes(p, 4)
                 blr += bytes([8, 0])    # blr_long
@@ -557,7 +576,7 @@ class WireProtocol:
             p.pack_int(0)
             send_channel(self.sock, p.get_buffer())
         else:
-            (blr, values) = self.params_to_blr(params)
+            (blr, values) = self.params_to_blr(trans_handle, params)
             p.pack_bytes(blr)
             p.pack_int(0)
             p.pack_int(1)
@@ -576,7 +595,7 @@ class WireProtocol:
             p.pack_int(0)
             send_channel(self.sock, p.get_buffer())
         else:
-            (blr, values) = self.params_to_blr(params)
+            (blr, values) = self.params_to_blr(trans_handle, params)
             p.pack_bytes(blr)
             p.pack_int(0)
             p.pack_int(1)
@@ -675,6 +694,15 @@ class WireProtocol:
         p.pack_int(0)
         send_channel(self.sock, p.get_buffer())
 
+    @wire_operation
+    def _op_put_segment(self, blob_handle, b):
+        p = xdrlib.Packer()
+        p.pack_int(self.op_put_segment)
+        p.pack_int(blob_handle)
+        p.pack_int(len(b))
+        p.pack_int(len(b))
+        send_channel(self.sock, p.get_buffer() + b)
+
     @wire_operation
     def _op_batch_segments(self, blob_handle, seg_data):
         ln = len(seg_data)
